/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#ifndef SESSION_CLEANUP_MGR_H
#define SESSION_CLEANUP_MGR_H

// This class is responsible for the timely destruction of sessions
// Its run method will run under a separate thread and will setup 
// timer at a high frequency compared to session expiry (few seconds). 
// At each timer expiry (or coming out of sleep) it will check the number of sessions to be 
// destroyed for that time and destroy them from thse SessionCacheManager.
// It will also provide interface to the SessionManager to register and update
// session expiry times

#include <string>
#include <map>
#include "ace/Reactor.h"
#include "ace\Synch.h"

struct ltlng
{
	bool operator()(long op1, long op2) const
	{
		return (op1 < op2);
	}
};

typedef std::multimap<long, std::string, ltlng> session_timeout_map;
typedef session_timeout_map::value_type  session_timeout_map_value_type;
typedef session_timeout_map::iterator session_timeout_map_iterator;
typedef std::pair <long, std::string> session_map_entry;

class SessionManager;
class SessionCacheManager;

class SessionCleanupManager : public ACE_Event_Handler
{
public:
	
	SessionCleanupManager();
	SessionCleanupManager(SessionManager *sessionManager, SessionCacheManager *sessionCache);
	

	virtual ~SessionCleanupManager();
	

	// expiry time (time as long since 1970 (same ol unix convention) ) !!!
	// May do some round of of time (something like to the next 5 seconds) 
	// which may be optimal in terms of resource usage for the timer
	void registerSessionExpiry(std::string sessionId, long expTime);
	
	// returns 0 on success
	int updateSessionExpiry(std::string sessionId, long currentExpTime, long newExpTime);

    // Need a good Timer class which is optimal in terms of resources
	// also. This class should probably use a single timer instance
	// to take care of all the timer needs. 

	// This interface may change - depends on the timer
	static void TimerCallBack(void *arg); 

	// Refresh session cache
	void refreshSessionCache();

	virtual int handle_timeout (const ACE_Time_Value &tv,
											const void *arg)
	{
		SessionCleanupManager::TimerCallBack((void*)this);
		return 1;
	}
private:
	// initialize the timer thread
	// void initTimer();

	SessionManager		*_sessionManager;
	SessionCacheManager *_sessionCache;
	session_timeout_map _sessionTimeoutMap;
	ACE_Thread_Mutex	_timeoutMapMutex;
};

#endif