/*
 * Copyright (c) 2002, Infosys Technologies Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution. 
 * 3) Neither the name of the Infosys Technologies Ltd. nor the names
 * of its contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#pragma warning ( disable : 4786 )

#include "gsoap_overwrite.h"

#include "MessageLogger.h"

// Callback functions for gsoap

static int http_response(struct soap *soap, int soap_error, size_t count) 
{
	MessageLogger::LogMessage('D', "SOAPServlet", "Entered http_response()");

    gsoapUserData *pRqConf = (gsoapUserData *) soap->user;
    if (NULL == pRqConf) {
	return SOAP_OK;
    }
    

    if (soap_error != SOAP_OK) {
		// See if need to call the sendError on HttpResponse from here
		// pRqConf->_srvltResp->sendError(404);
		return SOAP_HTTP_ERROR;
    }

	MessageLogger::LogMessage('D', "SOAPServlet", "Leaving http_response()");
    return SOAP_OK;
}

static int fsend(struct soap *psoap, const char *pBuf, size_t len) 
{
	MessageLogger::LogMessage('D', "SOAPServlet", "Entered fsend()");

    gsoapUserData *pRqConf = (gsoapUserData *)psoap->user;
    if (NULL == pRqConf) 
	{
		return SOAP_EOF;
    }

	ServletOutputStream & srvltOpStream = pRqConf->_srvltResp->getOutputStream();
	srvltOpStream.print(pBuf, len);

	MessageLogger::LogMessage('D', "SOAPServlet", "Leaving fsend()");
    return SOAP_OK;
}

static int reconstructHttpRequest(HttpServletRequest & httpSrvReq, std::string & httpReqStr)
{
	httpReqStr = httpSrvReq.getRequestLine();
	httpReqStr += "\r\n";

	// No need to add the headers as the function which parses the http 
	// headers can be overwritten to read from a different source

	// getting the body
	httpReqStr += httpSrvReq.getInputStream().str();
	return 0;
}

// The frecv function is called by the gsoap component to 
// get the header, body. There could be multiple calls so need to 
// keep track of what has been read in gsoapUserData struct.
// One more thing is that need to re-construct the entire request 
// (request line, headers and the body) from the HttpServletRequest object
// data members
static size_t frecv(struct soap *psoap, char *pBuf, size_t len) 
{
	MessageLogger::LogMessage('D', "SOAPServlet", "Entered frecv()");
    int nRet = 0;
    int nLenRemaining = len;
    char *pDest = pBuf;
    int nRemainingHeaderBytes = 0;

    gsoapUserData *pRqConf = (gsoapUserData *)psoap->user;
    if (NULL == pRqConf) 
	{
		return 0;
    }

	if( pRqConf->_httpReq == "")
	{
		pRqConf->_pos=0;
		if( 0 != reconstructHttpRequest(*pRqConf->_srvltReq, pRqConf->_httpReq))
			return 0;
	}

	nRemainingHeaderBytes = pRqConf->_httpReq.length() - pRqConf->_pos;
	std::string substr;
	int numToRead = 0;
	if(nRemainingHeaderBytes > 0)
	{
		if(nRemainingHeaderBytes >= len )
		{
			numToRead = len;
		}
		else
		{
			numToRead = nRemainingHeaderBytes;
		}
		substr = pRqConf->_httpReq.substr(pRqConf->_pos, numToRead);
		strncpy(pBuf, substr.c_str(), numToRead);
		pRqConf->_pos += numToRead;
	}


   MessageLogger::LogMessage('D', "SOAPServlet", "Leaving frecv()");
   return numToRead;
}


// get the headers and use psoap->fparsehdr()
// to make gsoap parse the headers 
static int http_parse(struct soap *psoap) 
{
	MessageLogger::LogMessage('D', "SOAPServlet", "Entered http_parse()");
    int nParse = SOAP_OK;
    gsoapUserData *pRqConf = NULL;
    if (NULL == psoap) 
	{
		return SOAP_EOF;
    }
    pRqConf = (gsoapUserData *)psoap->user;
    if (NULL == pRqConf) 
	{
		return SOAP_EOF;
    }

	HttpServletRequest *srvReqRef = pRqConf->_srvltReq;
	std::vector<std::string> hdrNames = srvReqRef->getHeaderNames();
	for(int i=0; i < hdrNames.size(); i++)
	{
		std::string hdrVal = srvReqRef->getHeader(hdrNames[i]);
		psoap->fparsehdr(psoap, hdrNames[i].c_str(), hdrVal.c_str());
	}

	MessageLogger::LogMessage('D', "SOAPServlet", "Leaving http_parse()");
    return SOAP_OK;
}


void set_callbacks(gsoapUserData *pRqConf, struct soap *psoap) 
{
    psoap->user = pRqConf;
    psoap->frecv = frecv;
    psoap->fsend = fsend;
    psoap->fresponse = http_response;
    psoap->fparse = http_parse;
}

